import json
from flask import Flask, request
from log import Log
from config import set_config
from recommend import video_recommend
from category import get_category_videos


app = Flask(__name__)
log_ = Log()
config_ = set_config()


@app.route('/healthcheck')
def health_check():
    return 'ok!'


# 首页推荐及tab分类
@app.route('/applet/video/homepage/recommend', methods=['GET', 'POST'])
def homepage_recommend():
    try:
        request_data = json.loads(request.get_data())
        mid = request_data.get('mid')
        uid = request_data.get('uid')
        category_id = request_data.get('categoryId')
        size = request_data.get('size')
        app_type = request_data.get('appType')
        algo_type = request_data.get('algoType')
        log_.info('request data: {}'.format(request_data))
        # size默认为10
        if not size:
            size = 10
        if category_id in config_.CATEGORY['recommend']:
            # 推荐
            videos = video_recommend()
            result = {'code': 200, 'message': 'success', 'data': {'videos': videos}}
            return json.dumps(result)
        elif category_id in config_.CATEGORY['other']:
            # 其他类别
            videos = get_category_videos()
            result = {'code': 200, 'message': 'success', 'data': {'videos': videos}}
            return json.dumps(result)
        else:
            log_.error('categoryId error, categoryId = {}'.format(category_id))
            result = {'code': -1, 'message': 'fail'}
            return json.dumps(result)

    except Exception as e:
        log_.error(e)
        result = {'code': -1, 'message': 'fail'}
        return json.dumps(result)


# 管理后台实时修改rov
@app.route('/applet/video/update/rov', methods=['GET', 'POST'])
def update_rov():
    pass


if __name__ == '__main__':
    app.run()
